@@ -1,8 +1,10 @@
+/* This software is distributed under the GNU Lesser General Public License */
 //==========================================================================
 //
 //	ratio_cut_partition.h
 //
 //==========================================================================
+// $Id: ratio_cut_partition.h,v 1.8 2003/01/31 08:15:04 chris Exp $
 
 #ifndef GTL_RATIO_CUT_PARTITION_H
 #define GTL_RATIO_CUT_PARTITION_H
@@ -32,18 +34,56 @@ class GTL_EXTERN ratio_cut_partition : public algorithm
 public:
     /**
      * Return type of @ref ratio_cut_partition#get_side_of_node.
+     *
+     * @see ratio_cut_partition#A
+     * @see ratio_cut_partition#B
      */
-    enum side_type {A = 0, B = 1};
+    typedef int side_type;
 
     /**
-     * Fix type of each node.
-     * <code>FIXA</code> means fixed on side <code>A</code>, <code>FIXB
-     * </code> on side <code>B</code> and <code>UNFIXED</code> means not
-     * fixed on any side.
+     * <code>A</code> means the node is on side A.
+     *
+     * @see ratio_cut_partition#side_type
+     */
+    const static side_type A;
+
+    /**
+     * <code>B</code> means the node is on side B.
+     *
+     * @see ratio_cut_partition#side_type
+     */
+    const static side_type B;
+
+    /**
+     * Fix type of each node (needed with
+     * @ref ratio_cut_partition#set_vars).
+     *
+     * @see ratio_cut_partition#FIXA
+     * @see ratio_cut_partition#FIXB
+     * @see ratio_cut_partition#UNFIXED
+     */
+    typedef short int fix_type;
+
+    /**
+     * <code>FIXA</code> means fix node on side <code>A</code>.
      *
      * @see ratio_cut_partition#set_vars
      */
-    enum fix_type {FIXA = 0, FIXB = 1, UNFIXED = 2};
+    const static fix_type FIXA;
+
+    /**
+     * <code>FIXB</code> means fix node on side <code>B</code>.
+     *
+     * @see ratio_cut_partition#fixe_type
+     */
+    const static fix_type FIXB;
+
+    /**
+     * <code>UNFIXED</code> means node is free.
+     *
+     * @see ratio_cut_partition#fixe_type
+     */
+    const static fix_type UNFIXED;
 
     /**
      * Default constructor.
@@ -65,9 +105,9 @@ public:
      * <code>source_node</code> and <code>target_node</code> will be
      * determined automatically.
      *
-     * @param <code>G</code> undirected graph.
-     * @param <code>node_weight</code> weight of each node.
-     * @param <code>edge_weight</code> weight of each edge.
+     * @param G undirected graph
+     * @param node_weight weight of each node
+     * @param edge_weight weight of each edge.
      * @see ratio_cut_partition#check
      */
     void set_vars(const graph& G, const node_map<int>& node_weight,
@@ -79,13 +119,11 @@ public:
      * In order to get good results, you should take two graph
      * theoretically far away nodes as source and target.
      *
-     * @param <code>G</code> undirected graph.
-     * @param <code>node_weight</code> weight of each node.
-     * @param <code>edge_weight</code> weight of each edge..
-     * @param <code>source_node</code> start-node, remains on side
-     * <code>A</code>.
-     * @param <code>target_node</code> end-node, remains on side <code>B
-     * </code>.
+     * @param G undirected graph
+     * @param node_weight weight of each node
+     * @param edge_weight weight of each edge
+     * @param source_node start-node, remains on side <code>A</code>
+     * @param target_node end-node, remains on side <code>B</code>
      * @see ratio_cut_partition#check
      */
     void set_vars(const graph& G, const node_map<int>& node_weight,
@@ -102,19 +140,17 @@ public:
      * init_side</code> and <code>target_node</code> on side <code>B
      * </code> respectively.
      *
-     * @param <code>G</code> undirected graph.
-     * @param <code>node_weight</code> weight of each node.
-     * @param <code>edge_weight</code> weight of each edge..
-     * @param <code>source_node</code> start-node, remains on side
-     * <code>A</code>.
-     * @param <code>target_node</code> end-node, remains on side <code>B
-     * </code>.
-     * @param <code>init_side</code> initial bi-partitioning.
+     * @param G undirected graph
+     * @param node_weight weight of each node
+     * @param edge_weight weight of each edge
+     * @param source_node start-node, remains on side <code>A</code>
+     * @param target_node end-node, remains on side <code>B</code>
+     * @param init_side initial bi-partitioning
      * @see ratio_cut_partition#check
      */
     void set_vars(const graph& G, const node_map<int>& node_weight,
 	const edge_map<int>& edge_weight, const node source_node,
-	const node target_node, const node_map<side_type> init_side);
+	const node target_node, const node_map<side_type>& init_side);
 			
     /**
      * Sets variables.
@@ -126,19 +162,17 @@ public:
      * <code>target_node</code> must not be fixed on side <code>A
      * </code>.
      *
-     * @param <code>G</code> undirected graph.
-     * @param <code>node_weight</code> weight of each node.
-     * @param <code>edge_weight</code> weight of each edge..
-     * @param <code>source_node</code> start-node, remains on side
-     * <code>A</code>.
-     * @param <code>target_node</code> end-node, remains on side <code>B
-     * </code>.
-     * @param <code>fixed</code> fixed nodes.
+     * @param G undirected graph
+     * @param node_weight weight of each node
+     * @param edge_weight weight of each edge
+     * @param source_node start-node, remains on side <code>A</code>
+     * @param target_node end-node, remains on side <code>B</code>
+     * @param fixed fixed nodes
      * @see ratio_cut_partition#check
      */
     void set_vars(const graph& G, const node_map<int>& node_weight,
 	const edge_map<int>& edge_weight, const node source_node,
-	const node target_node, const node_map<fix_type> fixed);
+	const node target_node, const node_map<fix_type>& fixed);
 
     /**
      * Sets variables.
@@ -154,29 +188,26 @@ public:
      * <code>target_node</code> must not be fixed on side <code>A
      * </code>.
      *
-     * @param <code>G</code> undirected graph.
-     * @param <code>node_weight</code> weight of each node.
-     * @param <code>edge_weight</code> weight of each edge..
-     * @param <code>source_node</code> start-node, remains on side
-     * <code>A</code>.
-     * @param <code>target_node</code> end-node, remains on side <code>B
-     * </code>.
-     * @param <code>init_side</code> initial bi-partitioning.
-     * @param <code>fixed</code> fixed nodes.
+     * @param G undirected graph
+     * @param node_weight weight of each node
+     * @param edge_weight weight of each edge
+     * @param source_node start-node, remains on side <code>A</code>
+     * @param target_node end-node, remains on side <code>B</code>
+     * @param init_side initial bi-partitioning
+     * @param fixed fixed nodes
      * @see ratio_cut_partition#check
      */
     void set_vars(const graph& G, const node_map<int>& node_weight,
 	const edge_map<int>& edge_weight, const node source_node,
-	const node target_node, const node_map<side_type> init_side,
-	const node_map<fix_type> fixed);
+	const node target_node, const node_map<side_type>& init_side,
+	const node_map<fix_type>& fixed);
 
     /**
      * Enables the storing of cut-edges. If enabled the list of
-     * cut-edges can be traversed using @ref ratio_cut_partition#
-     * cut_edges_iterator.
+     * cut-edges can be traversed using @ref
+     * ratio_cut_partition#cut_edges_iterator.
      *
-     * @param <code>set</code> if <code>true</code> cut_edges will be
-     * stored.
+     * @param set if <code>true</code> cut_edges will be stored
      * @see ratio_cut_partition#cut_edges_begin
      * @see ratio_cut_partition#cut_edges_end
      */
@@ -184,11 +215,10 @@ public:
 
     /**
      * Enables the storing of nodes on their side. If enabled the nodes
-     * of each side can be traversed using @ref ratio_cut_partition#
-     * nodes_on_one_side_iterator.
+     * of each side can be traversed using
+     * ratio_cut_partition#nodes_of_one_side_iterator.
      *
-     * @param <code>set</code> if <code>true</code> nodes on their side
-     * will be stored.
+     * @param set if <code>true</code> nodes on their side will be stored
      * @see ratio_cut_partition#nodes_of_sideA_begin
      * @see ratio_cut_partition#nodes_of_sideA_end
      * @see ratio_cut_partition#nodes_of_sideB_begin
@@ -214,9 +244,9 @@ public:
      * </code> is <code>FIXB</code>.
      * </ul>
      * 
-     * @param <code>G</code> graph
+     * @param G graph
      * @return <code>algorithm::GTL_OK</code> on success,
-     * <code>algorithm::GTL_ERROR</code> otherwise.
+     * <code>algorithm::GTL_ERROR</code> otherwise
      * @see ratio_cut_partition#set_vars
      * @see algorithm#check
      */
@@ -227,9 +257,9 @@ public:
      * of its vertices in two sides <code>ratio_cut_partition::A</code>
      * and <code>ratio_cut_partition::B</code>.
      * 
-     * @param <code>G</code> graph
+     * @param G graph
      * @return <code>algorithm::GTL_OK</code> on success,
-     * <code>algorithm::GTL_ERROR</code> otherwise.
+     * <code>algorithm::GTL_ERROR</code> otherwise
      * @see algorithm#run
      */
     int run(graph& G);
@@ -252,20 +282,20 @@ public:
     /**
      * Gets side of the node after bi-partitioning.
      * 
-     * @param <code>n</code> node of graph G.
+     * @param n node of graph G
      * @return <code>ratio_cut_partition::A</code> if <code>n</code>
      * lies on side <code>A</code>, <code>ratio_cut_partition::B</code>
-     * otherwise.
+     * otherwise
      */
     side_type get_side_of_node(const node& n) const;
 
     /**
      * Gets side of the node after bi-partitioning.
      * 
-     * @param <code>n</code> node of graph G.
+     * @param n node of graph G
      * @return <code>ratio_cut_partition::A</code> if <code>n</code>
      * lies on side <code>A</code>, <code>ratio_cut_partition::B</code>
-     * otherwise.
+     * otherwise
      * @see ratio_cut_partition#get_side_of_node
      */
     side_type operator [](const node& n) const;
@@ -274,7 +304,7 @@ public:
      * Gets the sum of all node weights from nodes on side <code>A
      * </code>.
      *
-     * @param <code>G</code> graph
+     * @param G graph
      * @return <code>node_weight_on_sideA</code>
      */
     int get_weight_on_sideA(const graph& G) const;
@@ -283,7 +313,7 @@ public:
      * Gets the sum of all node weights from nodes on side <code>B
      * </code>.
      *
-     * @param <code>G</code> graph
+     * @param G graph
      * @return <code>node_weight_on_sideB</code>
      */
     int get_weight_on_sideB(const graph& G) const;
@@ -296,20 +326,20 @@ public:
     /**
      * Iterate through all edges which belong to the cut, that means
      * all edges with end-nodes on different sides.
-     * It is only valid if enabled with @ref ratio_cut_partition#
-     * store_cut_edges before.
+     * It is only valid if enabled with @ref
+     * ratio_cut_partition#store_cut_edges before.
      *
-     * @return start for iteration through all cut edges.
+     * @return start for iteration through all cut edges
      */
     cut_edges_iterator cut_edges_begin() const;
 
     /**
      * End-Iterator for iteration through all edges which belong to the
      * cut.
-     * It is only valid if enabled with @ref ratio_cut_partition#
-     * store_cut_edges before.
+     * It is only valid if enabled with @ref
+     * ratio_cut_partition#store_cut_edges before.
      *
-     * @return end for iteration through all cut-edges.
+     * @return end for iteration through all cut-edges
      */
     cut_edges_iterator cut_edges_end() const;
 		
@@ -320,39 +350,39 @@ public:
 
     /**
      * Iterate through all nodes which belong to side <code>A</code>,
-     * It is only valid if enabled with @ref ratio_cut_partition#
-     * store_nodesAB before.
+     * It is only valid if enabled with @ref
+     * ratio_cut_partition#store_nodesAB before.
      *
-     * @return start for iteration through all nodes on <code>A</code>.
+     * @return start for iteration through all nodes on <code>A</code>
      */
     nodes_of_one_side_iterator nodes_of_sideA_begin() const;
 
     /**
      * End-Iterator for iteration through all nodes which belong to side
      * <code>A</code>,
-     * It is only valid if enabled with @ref ratio_cut_partition#
-     * store_nodesAB before.
+     * It is only valid if enabled with @ref
+     * ratio_cut_partition#store_nodesAB before.
      *
-     * @return end for iteration through all nodes on <code>A</code>.
+     * @return end for iteration through all nodes on <code>A</code>
      */
     nodes_of_one_side_iterator nodes_of_sideA_end() const;
 
     /**
      * Iterate through all nodes which belong to side <code>B</code>,
-     * It is only valid if enabled with @ref ratio_cut_partition#
-     * store_nodesAB before.
+     * It is only valid if enabled with @ref
+     * ratio_cut_partition#store_nodesAB before.
      *
-     * @return start for iteration through all nodes on <code>B</code>.
+     * @return start for iteration through all nodes on <code>B</code>
      */
     nodes_of_one_side_iterator nodes_of_sideB_begin() const;
 
     /**
      * End-Iterator for iteration through all nodes which belong to side
      * <code>B</code>,
-     * It is only valid if enabled with @ref ratio_cut_partition#
-     * store_nodesAB before.
+     * It is only valid if enabled with @ref
+     * ratio_cut_partition#store_nodesAB before.
      *
-     * @return end for iteration through all nodes on <code>B</code>.
+     * @return end for iteration through all nodes on <code>B</code>
      */
     nodes_of_one_side_iterator nodes_of_sideB_end() const;
 
@@ -652,14 +682,14 @@ protected:
     /**
      * @internal
      * Compute initial gain of a node on side <code>A</code>.
-     * @return initial gain_value of a node on side <code>A</code>.
+     * @return initial gain_value of a node on side <code>A</code>
      */
     int inital_gain_of_node_on_sideA(const node cur_node);
 
     /**
      * @internal
      * Compute initial gain of a node on side <code>B</code>.
-     * @return initial gain_value of a node on side <code>B</code>.
+     * @return initial gain_value of a node on side <code>B</code>
      */
     int inital_gain_of_node_on_sideB(const node cur_node);
 
@@ -706,7 +736,7 @@ protected:
      * Moves <code>max_gain</code> node from side <code>A</code> to
      * <code>B</code>.
      * @return <code>true</code> if vertex stored in parameter <code>
-     * moved_node</code> has been found.
+     * moved_node</code> has been found
      */
     bool move_vertex_A2B(const graph& G, node& moved_node);
 
@@ -714,7 +744,7 @@ protected:
      * @internal
      * Moves <code>max_gain node</code> from side B to A.
      * @return <code>true</code> if vertex stored in parameter <code>
-     * moved_node</code> has been found.
+     * moved_node</code> has been found
      */
     bool move_vertex_B2A(const graph& G, node& moved_node);
 
@@ -729,7 +759,7 @@ protected:
      * Computes <code>cut_ratio</code>.
      * @return <code>cut_ratio</code> with cutsize <code>cur_cutsize
      * </code> and current side weights <code>node_weight_on_sideA
-     * </code> and <code>node_weight_on_sideB</code>.
+     * </code> and <code>node_weight_on_sideB</code>
      */
     double cutratio();
 				
@@ -737,7 +767,7 @@ protected:
      * @internal
      * Corresponds to r(i) in [WeiChe91].
      * @return ratio gain of a node <code>cur_node</code> on side <code>
-     * A</code>.
+     * A</code>
      */
     double ratio_of_node_A2B(const node cur_node);
 
@@ -745,7 +775,7 @@ protected:
      * @internal
      * Corresponds to r(i) in [WeiChe91].
      * @return ratio gain of a node <code>cur_node</code> on side <code>
-     * B</code>.
+     * B</code>
      */
     double ratio_of_node_B2A(const node cur_node);
 		
@@ -831,7 +861,7 @@ protected:
      * @internal
      * Moves nodes in group swapping phase.
      * @return <code>true</code> on improvement, <code>false</code>
-     * else.
+     * else
      */
     bool move_manager(const graph& G);
 
@@ -839,7 +869,7 @@ protected:
      * @internal
      * Moves a single node.
      * @return <code>true</code> if vertex stored in parameter <code>
-     * moved_node</code> has been found.
+     * moved_node</code> has been found
      */
     bool move_vertex(const graph& G, node& moved_node);
 
@@ -855,7 +885,7 @@ protected:
      */
     void compute_nodesAB(const graph& G);
 private:
-#ifdef DEBUG
+#ifdef _DEBUG
     /**
      * @internal
      * Prints content of bucketA with associated gain values.
@@ -867,7 +897,7 @@ private:
      * Prints content of bucketB with associated gain values.
      */
     void print_bucketB();
-#endif	// DEBUG
+#endif	// _DEBUG
 };
 
 __GTL_END_NAMESPACE
